@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE StandaloneDeriving #-}
 {-# LANGUAGE GADTs #-}
 {-# LANGUAGE RankNTypes #-}
@@ -10,14 +11,14 @@
 -- 'withForeColor', 'withBackColor', and 'withStyle', e.g.,
 --
 -- @
---     defAttr `withForeColor` red
+--     defAttr \`withForeColor\` red
 -- @
 --
 -- 'Image' constructors often require an 'Attr' to indicate the
 -- attributes used in the image, e.g.,
 --
 -- @
---     string (defAttr `withForeColor` red) "this text will be red"
+--     string (defAttr \`withForeColor\` red) "this text will be red"
 -- @
 --
 -- The appearance of 'Image's using 'defAttr' is determined by the The
@@ -50,6 +51,9 @@ module Graphics.Vty.Attributes
   , withForeColor
   , withBackColor
 
+  -- * Setting hyperlinks
+  , withURL
+
   -- * Colors
   , module Graphics.Vty.Attributes.Color
   , module Graphics.Vty.Attributes.Color240
@@ -57,6 +61,10 @@ module Graphics.Vty.Attributes
 where
 
 import Data.Bits
+#if !(MIN_VERSION_base(4,11,0))
+import Data.Semigroup
+#endif
+import Data.Text (Text)
 import Data.Word
 
 import Graphics.Vty.Attributes.Color
@@ -73,6 +81,7 @@ data Attr = Attr
     { attrStyle :: !(MaybeDefault Style)
     , attrForeColor :: !(MaybeDefault Color)
     , attrBackColor :: !(MaybeDefault Color)
+    , attrURL :: !(MaybeDefault Text)
     } deriving ( Eq, Show, Read )
 
 -- This could be encoded into a single 32 bit word. The 32 bit word is
@@ -106,12 +115,18 @@ data Attr = Attr
 --  Then the foreground color encoded into 8 bits.
 --  Then the background color encoded into 8 bits.
 
+instance Semigroup Attr where
+    attr0 <> attr1 =
+        Attr ( attrStyle attr0     <> attrStyle attr1 )
+             ( attrForeColor attr0 <> attrForeColor attr1 )
+             ( attrBackColor attr0 <> attrBackColor attr1 )
+             ( attrURL attr0       <> attrURL attr1 )
+
 instance Monoid Attr where
-    mempty = Attr mempty mempty mempty
-    mappend attr0 attr1 =
-        Attr ( attrStyle attr0     `mappend` attrStyle attr1 )
-             ( attrForeColor attr0 `mappend` attrForeColor attr1 )
-             ( attrBackColor attr0 `mappend` attrBackColor attr1 )
+    mempty = Attr mempty mempty mempty mempty
+#if !(MIN_VERSION_base(4,11,0))
+    mappend = (<>)
+#endif
 
 -- | Specifies the display attributes such that the final style and
 -- color values do not depend on the previously applied display
@@ -121,6 +136,7 @@ data FixedAttr = FixedAttr
     { fixedStyle :: !Style
     , fixedForeColor :: !(Maybe Color)
     , fixedBackColor :: !(Maybe Color)
+    , fixedURL       :: !(Maybe Text)
     } deriving ( Eq, Show )
 
 -- | The style and color attributes can either be the terminal defaults.
@@ -135,17 +151,22 @@ deriving instance Eq v => Eq (MaybeDefault v)
 deriving instance Eq v => Show (MaybeDefault v)
 deriving instance (Eq v, Show v, Read v) => Read (MaybeDefault v)
 
+instance Eq v => Semigroup (MaybeDefault v) where
+    Default     <> Default     = Default
+    Default     <> KeepCurrent = Default
+    Default     <> SetTo v     = SetTo v
+    KeepCurrent <> Default     = Default
+    KeepCurrent <> KeepCurrent = KeepCurrent
+    KeepCurrent <> SetTo v     = SetTo v
+    SetTo _v    <> Default     = Default
+    SetTo v     <> KeepCurrent = SetTo v
+    SetTo _     <> SetTo v     = SetTo v
+
 instance Eq v => Monoid ( MaybeDefault v ) where
     mempty = KeepCurrent
-    mappend Default Default = Default
-    mappend Default KeepCurrent = Default
-    mappend Default ( SetTo v ) = SetTo v
-    mappend KeepCurrent Default = Default
-    mappend KeepCurrent KeepCurrent = KeepCurrent
-    mappend KeepCurrent ( SetTo v ) = SetTo v
-    mappend ( SetTo _v ) Default = Default
-    mappend ( SetTo v ) KeepCurrent = SetTo v
-    mappend ( SetTo _ ) ( SetTo v ) = SetTo v
+#if !(MIN_VERSION_base(4,11,0))
+    mappend = (<>)
+#endif
 
 -- | Styles are represented as an 8 bit word. Each bit in the word is 1
 -- if the style attribute assigned to that bit should be applied and 0
@@ -200,13 +221,28 @@ withBackColor attr c = attr { attrBackColor = SetTo c }
 
 -- | Add the given style attribute
 withStyle :: Attr -> Style -> Attr
+withStyle attr 0 = attr
 withStyle attr styleFlag = attr { attrStyle = SetTo $ styleMask attr .|. styleFlag }
 
+-- | Add a hyperlinked URL using the proposed [escape sequences for
+-- hyperlinked
+-- URLs](https://gist.github.com/egmontkob/eb114294efbcd5adb1944c9f3cb5feda).
+-- These escape sequences are comparatively new and aren't widely
+-- supported in terminal emulators yet, but most terminal emulators
+-- that don't know about these sequences will ignore these sequences,
+-- and therefore this should fall back sensibly. In some cases they
+-- won't and this will result in garbage, so this is why hyperlinking is
+-- disabled by default, in which case this combinator has no observable
+-- effect. To enable it, enable 'Hyperlink' mode on your Vty output
+-- interface.
+withURL :: Attr -> Text -> Attr
+withURL attr url = attr { attrURL = SetTo url }
+
 -- | Sets the style, background color and foreground color to the
 -- default values for the terminal. There is no easy way to determine
 -- what the default background and foreground colors are.
 defAttr :: Attr
-defAttr = Attr Default Default Default
+defAttr = Attr Default Default Default Default
 
 -- | Keeps the style, background color and foreground color that was
 -- previously set. Used to override some part of the previous style.
@@ -216,4 +252,4 @@ defAttr = Attr Default Default Default
 -- Would be the currently applied style (be it underline, bold, etc) but
 -- with the foreground color set to brightMagenta.
 currentAttr :: Attr
-currentAttr = Attr KeepCurrent KeepCurrent KeepCurrent
+currentAttr = Attr KeepCurrent KeepCurrent KeepCurrent KeepCurrent
