@@ -1,5 +1,6 @@
 {-# LANGUAGE RecordWildCards #-}
 {-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE OverloadedStrings #-}
 {-# OPTIONS_GHC -D_XOPEN_SOURCE=500 -fno-warn-warnings-deprecations #-}
 {-# CFILES gwinsz.c #-}
 
@@ -12,6 +13,7 @@ module Graphics.Vty.Output.TerminfoBased
 where
 
 import Control.Monad (when)
+import qualified Data.ByteString as BS
 import Data.ByteString.Internal (toForeignPtr)
 import Data.Terminfo.Parse
 import Data.Terminfo.Eval
@@ -21,11 +23,12 @@ import Graphics.Vty.Image (DisplayRegion)
 import Graphics.Vty.DisplayAttributes
 import Graphics.Vty.Output.Interface
 
-import Blaze.ByteString.Builder (Write, writeToByteString)
+import Blaze.ByteString.Builder (Write, writeToByteString, writeStorable)
 
 import Control.Monad.Trans
 
 import Data.Bits ((.&.))
+import Data.Foldable (foldMap)
 import Data.IORef
 import Data.Maybe (isJust, isNothing, fromJust)
 import Data.Word
@@ -117,6 +120,25 @@ reserveTerminal termName outFd = liftIO $ do
                     Just setb -> setb
                     Nothing -> error $ "no back color support for terminal " ++ termName
                 Just setab -> setab
+
+    hyperlinkModeStatus <- newIORef False
+    newAssumedStateRef <- newIORef initialAssumedState
+
+    let terminfoSetMode m newStatus = do
+          curStatus <- terminfoModeStatus m
+          when (newStatus /= curStatus) $
+              case m of
+                  Hyperlink -> liftIO $ do
+                      writeIORef hyperlinkModeStatus newStatus
+                      writeIORef newAssumedStateRef initialAssumedState
+                  _ -> return ()
+        terminfoModeStatus m =
+            case m of
+                Hyperlink -> liftIO $ readIORef hyperlinkModeStatus
+                _ -> return False
+        terminfoModeSupported Hyperlink = True
+        terminfoModeSupported _ = False
+
     terminfoCaps <- pure TerminfoCaps
         <*> probeCap ti "smcup"
         <*> probeCap ti "rmcup"
@@ -132,7 +154,6 @@ reserveTerminal termName outFd = liftIO $ do
         <*> requireCap ti "el"
         <*> currentDisplayAttrCaps ti
         <*> probeCap ti "bel"
-    newAssumedStateRef <- newIORef initialAssumedState
     let t = Output
             { terminalID = termName
             , releaseTerminal = liftIO $ do
@@ -168,9 +189,9 @@ reserveTerminal termName outFd = liftIO $ do
                         Just v -> toEnum v
                     True -> 1
             , supportsCursorVisibility = isJust $ civis terminfoCaps
-            , supportsMode = const False
-            , setMode = const $ const $ return ()
-            , getModeStatus = const $ return False
+            , supportsMode = terminfoModeSupported
+            , setMode = terminfoSetMode
+            , getModeStatus = terminfoModeStatus
             , assumedStateRef = newAssumedStateRef
             -- I think fix would help assure tActual is the only
             -- reference. I was having issues tho.
@@ -232,11 +253,27 @@ terminfoDisplayContext tActual terminfoCaps r = return dc
                 Nothing -> error "this terminal does not support hide cursor"
                 Just c -> writeCapExpr c []
             , writeSetAttr = terminfoWriteSetAttr dc terminfoCaps
-            , writeDefaultAttr = writeCapExpr (setDefaultAttr terminfoCaps) []
+            , writeDefaultAttr = \urlsEnabled ->
+                writeCapExpr (setDefaultAttr terminfoCaps) [] `mappend`
+                (if urlsEnabled then writeURLEscapes EndLink else mempty)
             , writeRowEnd = writeCapExpr (clearEol terminfoCaps) []
             , inlineHack = return ()
             }
 
+-- | Write the escape sequences that are used in some terminals to
+-- include embedded hyperlinks. As of yet, this information isn't
+-- included in termcap or terminfo, so this writes them directly
+-- instead of looking up the appropriate capabilities.
+writeURLEscapes :: URLDiff -> Write
+writeURLEscapes (LinkTo url) =
+    foldMap writeStorable (BS.unpack "\x1b]8;;") `mappend`
+    foldMap writeStorable (BS.unpack url) `mappend`
+    writeStorable (0x07 :: Word8)
+writeURLEscapes EndLink =
+    foldMap writeStorable (BS.unpack "\x1b]8;;\a")
+writeURLEscapes NoLinkChange =
+    mempty
+
 -- | Portably setting the display attributes is a giant pain in the ass.
 --
 -- If the terminal supports the sgr capability (which sets the on/off
@@ -273,9 +310,9 @@ terminfoDisplayContext tActual terminfoCaps r = return dc
 --
 -- Note that this optimizes for fewer state changes followed by fewer
 -- bytes.
-terminfoWriteSetAttr :: DisplayContext -> TerminfoCaps -> FixedAttr -> Attr -> DisplayAttrDiff -> Write
-terminfoWriteSetAttr dc terminfoCaps prevAttr reqAttr diffs = do
-    case (foreColorDiff diffs == ColorToDefault) || (backColorDiff diffs == ColorToDefault) of
+terminfoWriteSetAttr :: DisplayContext -> TerminfoCaps -> Bool -> FixedAttr -> Attr -> DisplayAttrDiff -> Write
+terminfoWriteSetAttr dc terminfoCaps urlsEnabled prevAttr reqAttr diffs = do
+    urlAttrs urlsEnabled `mappend` case (foreColorDiff diffs == ColorToDefault) || (backColorDiff diffs == ColorToDefault) of
         -- The only way to reset either color, portably, to the default
         -- is to use either the set state capability or the set default
         -- capability.
@@ -284,7 +321,7 @@ terminfoWriteSetAttr dc terminfoCaps prevAttr reqAttr diffs = do
                                      (fixedStyle attr )
                                      (styleToApplySeq $ fixedStyle attr) of
                 -- only way to reset a color to the defaults
-                EnterExitSeq caps -> writeDefaultAttr dc
+                EnterExitSeq caps -> writeDefaultAttr dc urlsEnabled
                                      `mappend`
                                      foldMap (\cap -> writeCapExpr cap []) caps
                                      `mappend`
@@ -324,6 +361,8 @@ terminfoWriteSetAttr dc terminfoCaps prevAttr reqAttr diffs = do
                                                (sgrArgsForState state)
                                   `mappend` setColors
     where
+        urlAttrs True = writeURLEscapes (urlDiff diffs)
+        urlAttrs False = mempty
         colorMap = case useAltColorMap terminfoCaps of
                         False -> ansiColorIndex
                         True -> altColorIndex
