@@ -4,18 +4,17 @@ import Control.Applicative ((<$>))
 
 import Graphics.Vty.Output.Mock
 
-import qualified Data.ByteString as BS
 import Data.IORef
 import qualified Data.String.UTF8 as UTF8
 
 import Test.QuickCheck.Property
 
--- A list of terminals that should be supported.
--- This started with a list of terminals ubuntu supported. Then those terminals that really could
--- not be supported were removed. Then a few more were pruned until a reasonable looking set was
--- made.
+-- A list of terminals that should be supported. This started with a
+-- list of terminals ubuntu supported. Then those terminals that really
+-- could not be supported were removed. Then a few more were pruned
+-- until a reasonable looking set was made.
 terminalsOfInterest :: [String]
-terminalsOfInterest = 
+terminalsOfInterest =
     [ "vt100"
     , "vt220"
     , "vt102"
@@ -58,4 +57,3 @@ compareMockOutput mockData expectedStr = do
                                       ++ expectedStr
                              }
         else return succeeded
-
